/**
1   ascending order on sum of the first two elements, use first element as a tie breaker
2  descending order on sum of the first two elements, use first element as a tie breaker
3   ascending order on sum of the first and third elements, use first element as a tie breaker
4   ascending order on maximum of the second and third elements, use first element as a tie breaker
5  descending order on maximum of the second and third elements, use first element as a tie breaker
6   ascending order on minimum of the first and second elements, use third element as a tie breaker
7   ascending order on first element, use second element as a tie breaker
8  descending order on third element, use first element as a tie breaker
9   ascending order on first element, use second element, in descending order, as a tie breaker
10  ascending order on product of the first two elements, use third element as a tie breaker
*/
import java.util.*;
import java.util.Random;
import java.util.function.*;

public class Triples
{
   public static void main(String[] args)
   {
      List<Triple> triples = Triple.makeTriples(12, 10);

      for (Triple t : triples)
         System.out.print( t );
      System.out.println("\n");

      // Give nice names to some of the lambdas that we need.
      // The types all come from java.util.function
      IntBinaryOperator prod = (x,y) -> x * y;
      BiPredicate<Integer, Integer> lt = (x,y) -> x < y;
      BiPredicate<Integer, Integer> gt = (x,y) -> x > y;

      // Code to sort triples ten different ways.
/*1*/ triples.stream().sorted(new CmpTriples(Triple::first, Triple::second, Integer::sum, lt, Triple::first, lt))
      .forEach(System.out::print);
      System.out.println();

/*2*/ triples.stream().sorted(new CmpTriples(Triple::first, Triple::second, Integer::sum, gt, Triple::first, gt))
      .forEach(System.out::print);
      System.out.println();

/*3*/ triples.stream().sorted(new CmpTriples(Triple::first, Triple::third, Integer::sum, lt, Triple::first, lt))
      .forEach(System.out::print);
      System.out.println();

/*4*/ triples.stream().sorted(new CmpTriples(Triple::second, Triple::third, Math::max, lt, Triple::first, lt))
      .forEach(System.out::print);
      System.out.println();

/*5*/ triples.stream().sorted(new CmpTriples(Triple::second, Triple::third, Math::max, gt, Triple::first, gt))
      .forEach(System.out::print);
      System.out.println();

/*6*/ triples.stream().sorted(new CmpTriples(Triple::first, Triple::second, Math::min, lt, Triple::third, lt))
      .forEach(System.out::print);
      System.out.println();

/*7*/ triples.stream().sorted(new CmpTriples(Triple::first, Triple::first, Math::max, lt, Triple::second, lt))
      .forEach(System.out::print);
      System.out.println();

/*8*/ triples.stream().sorted(new CmpTriples(Triple::third, Triple::third, Math::max, gt, Triple::first, gt))
      .forEach(System.out::print);
      System.out.println();

/*9*/ triples.stream().sorted(new CmpTriples(Triple::first, Triple::first, Math::max, lt, Triple::second, gt))
      .forEach(System.out::print);
      System.out.println();

/*10*/triples.stream().sorted(new CmpTriples(Triple::first, Triple::second, prod, lt, Triple::third, lt))
      .forEach(System.out::print);
      System.out.println();
   }//main()
}//Triples



class CmpTriples implements Comparator<Triple>
{  // These paramaters are all "functional interfaces"
   private ToIntFunction<Triple> s1;
   private ToIntFunction<Triple> s2;
   private IntBinaryOperator reduce;
   private BiPredicate<Integer, Integer> cmp1;
   private ToIntFunction<Triple> s3;
   private BiPredicate<Integer, Integer> cmp2;

   CmpTriples(ToIntFunction<Triple> s1,
              ToIntFunction<Triple> s2,
              IntBinaryOperator reduce,
              BiPredicate<Integer, Integer> cmp1,
              ToIntFunction<Triple> s3,
              BiPredicate<Integer, Integer> cmp2)
   {
      this.s1 = s1;
      this.s2 = s2;
      this.reduce = reduce;
      this.cmp1 = cmp1;
      this.s3 = s3;
      this.cmp2 = cmp2;
   }

   public int compare(Triple t1, Triple t2)
   {  // this is correct only for strict inequalities, < and >
      int n1 = reduce.applyAsInt(s1.applyAsInt(t1), s2.applyAsInt(t1));
      int n2 = reduce.applyAsInt(s1.applyAsInt(t2), s2.applyAsInt(t2));
      int n3 = s3.applyAsInt(t1);
      int n4 = s3.applyAsInt(t2);
      if (cmp1.test(n1, n2)) return -1;
      if (cmp1.test(n2, n1)) return  1;
      if (cmp2.test(n3, n4)) return -1;
      if (cmp2.test(n4, n3)) return  1;
      return 0;
   }
}//CmpTriples



class Triple
{
   private int first;
   private int second;
   private int third;

   private static Random generator = new Random();

   public Triple(int first, int second, int third)
   {
      this.first  = first;
      this.second = second;
      this.third  = third;
   }

   public int first() { return first; }
   public int second(){ return second; }
   public int third() { return third; }

   public String toString()
   {
      return "(" + first + " " + second + " " + third + ")";
   }

   public static List<Triple> makeTriples(int n1, int n2)
   {
      List<Triple> result = new ArrayList<Triple>();
      for (int i = 0; i < n1; i++)
      {
         int r1 = generator.nextInt(n2);
         int r2 = generator.nextInt(n2);
         int r3 = generator.nextInt(n2);
         Triple triple = new Triple(r1, r2, r3);
         result.add(triple);
      }
      return result;
   }
}//Triple
