import Data.List

-- https://hackage.haskell.org/package/base-4.12.0.0/docs/Data-List.html#v:sort
-- https://hackage.haskell.org/package/base-4.12.0.0/docs/Data-List.html#v:sortBy
-- https://hackage.haskell.org/package/base-4.12.0.0/docs/Data-Ord.html#t:Ordering

data Triple = T Integer Integer Integer deriving Show

-- Selector functions for Triple
e1 :: Triple -> Integer
e2 :: Triple -> Integer
e3 :: Triple -> Integer
e1 (T x _ _) = x
e2 (T _ y _) = y
e3 (T _ _ z) = z

-- Types for making it easier to define makeCmp.
type Comparator = Triple -> Triple -> Ordering
type Selector   = Triple -> Integer
type Reduce     = Integer -> Integer -> Integer
type Cmp        = Integer -> Integer -> Bool

makeCmp :: Selector -> Selector -> Reduce -> Cmp -> Selector -> Cmp -> Comparator
makeCmp s1 s2 r cmp1 s3 cmp2 = comparator
   where comparator t1 t2 | cmp1 n1 n2 = LT
                          | cmp1 n2 n1 = GT
                          | cmp2 n3 n4 = LT
                          | cmp2 n4 n3 = GT
                          | otherwise  = EQ
                          where n1 = r (s1 t1) (s2 t1)
                                n2 = r (s1 t2) (s2 t2)
                                n3 = (s3 t1)
                                n4 = (s3 t2)

data0 = [(T 3 2 4),(T 4 1 9),(T 7 6 1),(T 7 8 1),(T 0 6 4),(T 5 9 0),
         (T 4 1 6),(T 4 5 5),(T 2 8 5),(T 5 9 3),(T 3 2 9),(T 4 2 3)]

--1  ascending order on sum of the first two elements, use first element as a tie breaker
--2 descending order on sum of the first two elements, use first element as a tie breaker
--3  ascending order on sum of the first and third elements, use first element as a tie breaker
--4  ascending order on maximum of the second and third elements, use first element as a tie breaker
--5 descending order on maximum of the second and third elements, use first element as a tie breaker
--6  ascending order on minimum of the first and second elements, use third element as a tie breaker
--7  ascending order on first element, use second element as a tie breaker
--8 descending order on third element, use first element as a tie breaker
--9  ascending order on first element, use second element, in descending order, as a tie breaker
--10 ascending order on product of the first two elements, use third element as a tie breaker

data1  = sortBy (makeCmp e1 e2 (+) (<) e1 (<)) data0
data2  = sortBy (makeCmp e1 e2 (+) (>) e1 (>)) data0
data3  = sortBy (makeCmp e1 e3 (+) (<) e1 (<)) data0
data4  = sortBy (makeCmp e2 e3 max (<) e1 (<)) data0
data5  = sortBy (makeCmp e2 e3 max (>) e1 (>)) data0
data6  = sortBy (makeCmp e1 e2 min (<) e3 (<)) data0
data7  = sortBy (makeCmp e1 e1 max (<) e2 (<)) data0
data8  = sortBy (makeCmp e3 e3 max (>) e1 (>)) data0
data9  = sortBy (makeCmp e1 e1 max (<) e2 (>)) data0
data10 = sortBy (makeCmp e1 e2 (*) (<) e3 (<)) data0

main = do
  print data0
  putStrLn ""
  print data1
  print data2
  print data3
  print data4
  print data5
  print data6
  print data7
  print data8
  print data9
  print data10

{-
(3 2 4)(4 1 9)(7 6 1)(7 8 1)(0 6 4)(5 9 0)(4 1 6)(4 5 5)(2 8 5)(5 9 3)(3 2 9)(4 2 3)

(3 2 4)(3 2 9)(4 1 9)(4 1 6)(0 6 4)(4 2 3)(4 5 5)(2 8 5)(7 6 1)(5 9 0)(5 9 3)(7 8 1)
(7 8 1)(5 9 0)(5 9 3)(7 6 1)(2 8 5)(4 5 5)(4 2 3)(0 6 4)(4 1 9)(4 1 6)(3 2 4)(3 2 9)
(0 6 4)(5 9 0)(2 8 5)(3 2 4)(4 2 3)(5 9 3)(7 6 1)(7 8 1)(4 5 5)(4 1 6)(3 2 9)(4 1 9)
(4 2 3)(3 2 4)(4 5 5)(0 6 4)(4 1 6)(7 6 1)(2 8 5)(7 8 1)(3 2 9)(4 1 9)(5 9 0)(5 9 3)
(5 9 0)(5 9 3)(4 1 9)(3 2 9)(7 8 1)(2 8 5)(7 6 1)(4 1 6)(0 6 4)(4 5 5)(3 2 4)(4 2 3)
(0 6 4)(4 1 6)(4 1 9)(4 2 3)(3 2 4)(2 8 5)(3 2 9)(4 5 5)(5 9 0)(5 9 3)(7 6 1)(7 8 1)
(0 6 4)(2 8 5)(3 2 4)(3 2 9)(4 1 9)(4 1 6)(4 2 3)(4 5 5)(5 9 0)(5 9 3)(7 6 1)(7 8 1)
(4 1 9)(3 2 9)(4 1 6)(4 5 5)(2 8 5)(3 2 4)(0 6 4)(5 9 3)(4 2 3)(7 6 1)(7 8 1)(5 9 0)
(0 6 4)(2 8 5)(3 2 4)(3 2 9)(4 5 5)(4 2 3)(4 1 9)(4 1 6)(5 9 0)(5 9 3)(7 8 1)(7 6 1)
(0 6 4)(4 1 6)(4 1 9)(3 2 4)(3 2 9)(4 2 3)(2 8 5)(4 5 5)(7 6 1)(5 9 0)(5 9 3)(7 8 1)
-}
